SELECT
  COUNT(DISTINCT sha256) AS conflicts,
  pname.str AS pname,
  ver.str AS ver,
  plat.str AS plat,
  GROUP_CONCAT(name.str, char(10)) AS name,
  GROUP_CONCAT(IFNULL(tag.str, char(0x274C)), char(10)) AS tag,
  GROUP_CONCAT(h.hash, char(10)) AS sha256
FROM
  (
    CudaArtifact AS cc,
    Str AS name,
    Str AS pname,
    Str as ver,
    Str as plat,
    Hash as h ON cc.name = name.id
    AND cc.pname = pname.id
    AND cc.version = ver.id
    AND cc.platform = plat.id
    AND cc.sha256 = h.id
  )
  LEFT JOIN Str AS tag ON cc.compat_tag = tag.id
GROUP BY
  cc.pname,
  cc.version,
  cc.platform
HAVING
  conflicts >= CAST(:min_conflicts AS INTEGER)
ORDER BY
  conflicts DESC,
  cc.pname,
  cc.version,
  cc.platform,
  tag.str
