WITH PerQuadruple AS (
  SELECT
    pname.str AS pname,
    IFNULL(ver.str, char(0x274C)) AS version,
    plat.str AS platform,
    IFNULL(ct.str, char(0x274C)) AS tag,
    COUNT(DISTINCT sha256.hash) AS uq_sha256s
  FROM
    (
      CudaArtifact AS ca,
      Str AS pname,
      Str AS plat,
      Hash AS sha256 ON ca.pname = pname.id
      AND ca.platform = plat.id
      AND sha256.id = ca.sha256
    )
    LEFT JOIN Str AS ver ON ver.id = ca.version
    LEFT JOIN Str AS ct ON ca.compat_tag = ct.id
  GROUP BY
    pname.id,
    version,
    platform,
    tag
  ORDER BY
    uq_sha256s DESC,
    pname.str,
    version,
    plat.str,
    tag
),
ByPname AS (
  SELECT
    pname,
    REPLACE(GROUP_CONCAT(DISTINCT pq.version), ',', char(10)) AS versions,
    REPLACE(
      GROUP_CONCAT(DISTINCT pq.platform),
      ',',
      char(10)
    ) AS platforms,
    REPLACE(GROUP_CONCAT(DISTINCT pq.tag), ',', char(10)) AS tags,
    SUM(pq.uq_sha256s) AS uq_sha256s
  FROM
    PerQuadruple AS pq
  GROUP BY
    pq.pname
),
VersionGroups AS (
  SELECT
    REPLACE(GROUP_CONCAT(DISTINCT pname), ',', char(10)) AS pnames,
    versions
  FROM
    ByPname AS bp
  GROUP BY
    versions
  ORDER BY
    LENGTH(pnames) DESC
),
PlatformGroups AS (
  SELECT
    REPLACE(GROUP_CONCAT(DISTINCT pname), ',', char(10)) AS pnames,
    platforms
  FROM
    ByPname AS bp
  GROUP BY
    platforms
  ORDER BY
    LENGTH(platforms) DESC,
    LENGTH(pnames)
)
SELECT
  *
FROM
  PlatformGroups
