WITH PerQuadruple AS (
  SELECT
    pname.str AS pname,
    IFNULL(ver.str, char(0x274C)) AS version,
    plat.str AS platform,
    IFNULL(ct.str, char(0x274C)) AS tag,
    COUNT(DISTINCT sha256.hash) AS uq_sha256s
  FROM
    (
      CudaArtifact AS ca,
      Str AS pname,
      Str AS plat,
      Hash AS sha256 ON ca.pname = pname.id
      AND ca.platform = plat.id
      AND sha256.id = ca.sha256
    )
    LEFT JOIN Str AS ver ON ver.id = ca.version
    LEFT JOIN Str AS ct ON ca.compat_tag = ct.id
  GROUP BY
    pname.id,
    version,
    platform,
    tag
  ORDER BY
    uq_sha256s DESC,
    pname.str,
    version,
    plat.str,
    tag
)
SELECT
  pname,
  (
    SELECT
      GROUP_CONCAT(x, char(10))
    FROM
      (
        SELECT
          DISTINCT version AS x
        FROM
          PerQuadruple
        WHERE
          pname = pq.pname
        ORDER BY
          x
      )
  ) AS versions,
  (
    SELECT
      GROUP_CONCAT(x, char(10))
    FROM
      (
        SELECT
          DISTINCT platform AS x
        FROM
          PerQuadruple
        WHERE
          pname = pq.pname
        ORDER BY
          x
      )
  ) AS platforms,
  (
    SELECT
      GROUP_CONCAT(x, char(10))
    FROM
      (
        SELECT
          DISTINCT tag AS x
        FROM
          PerQuadruple
        WHERE
          pname = pq.pname
        ORDER BY
          x
      )
  ) AS tags,
  (
    SELECT
      SUM(uq_sha256s)
    FROM
      PerQuadruple
    WHERE
      pname = pq.pname
  ) AS uq_sha256s
FROM
  PerQuadruple AS pq
